import importlib.util
import logging
import os

logger = logging.getLogger(__name__)


class UserConfig:
    config_list = []
    theme = None


def read_user_config():

    config_path = os.path.join(
        os.environ["XDG_CONFIG_HOME"]
        if "XDG_CONFIG_HOME" in os.environ
        else os.path.join(os.environ["HOME"], ".config"),
        "i3",
        "i3_notifier_config.py",
    )

    if os.path.exists(config_path):
        logger.info(f"Loading user config from {config_path}")
        spec = importlib.util.spec_from_file_location("i3_notifier_config", config_path)
        userconfig = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(userconfig)
        return userconfig
    logger.info(f"File not found: {config_path}")


def get_user_config():
    userconfig = UserConfig()
    userconfig_ = read_user_config()

    if userconfig_ is not None:
        if hasattr(userconfig_, "config_list"):
            userconfig.config_list = userconfig_.config_list

        if hasattr(userconfig_, "theme"):
            userconfig.theme = userconfig_.theme

    return userconfig



import unittest

from i3notifier.notification import Notification, NotificationCluster


class TestNotificaton(unittest.TestCase):

    _notification = Notification(
        1, "app", "icon", "summary", "body", ["default"], 1595608250375722880, 1
    )

    def test_format(self):
        self.assertEqual(
            self._notification.formatted(),
            b"<b>summary</b> <small>app</small>\n<i>body</i>",
        )


if __name__ == "__main__":
    unittest.main()



import html
import threading
from html.parser import HTMLParser
from io import StringIO


class MLStripper(HTMLParser):
    def __init__(self):
        super().__init__()
        self.reset()
        self.strict = False
        self.convert_charrefs = True
        self.text = StringIO()

    def handle_data(self, d):
        self.text.write(d)

    def get_data(self):
        return self.text.getvalue()


def strip_tags(data):
    s = MLStripper()
    s.feed(html.escape(data))
    return s.get_data()


class RunAsync(threading.Thread):

    __slots__ = "args", "kwargs", "closure"

    def __init__(self, closure, *args, **kwargs):
        self.closure = closure
        self.args = args
        self.kwargs = kwargs
        threading.Thread.__init__(self)

    def run(self):
        return self.closure(*self.args, **self.kwargs)


class RunAsyncFactory:
    __slots__ = "closure"

    def __init__(self, closure):
        self.closure = closure

    def __call__(self, *args, **kwargs):
        return RunAsync(self.closure, *args, **kwargs).start()



import os.path
import subprocess
from enum import Enum


class Operation(Enum):
    SELECT = 0
    EXIT = 1
    DELETE = 10


class RofiGUI:

    _separator = b"\x01"
    _args = [
        "-dmenu",
        "-markup-rows",
        "-i",
        "-format",
        "i",
        "-sep",
        r"\x01",
    ]

    __slots__ = "cmd"

    def __init__(self, *args, theme=None, cmd=None):
        self.cmd = cmd or "rofi"
        self._args.extend(args)
        if theme is not None:
            self._args.extend(
                ["-theme", f"{os.path.dirname(__file__)}/rofi-theme/{theme}"]
            )

    def show_notifications(self, notifications, row=0):

        formatted_notifications = []
        urgent = []
        active = []
        for i, notification in enumerate(notifications):
            formatted_notifications.append(notification.formatted())

            if notification.urgency == 2:
                urgent.append(str(i))

            if len(notification) > 1:
                active.append(str(i))

        proc = subprocess.Popen(
            [self.cmd]
            + self._args
            + ["-selected-row", str(row)]
            + (["-u", ",".join(urgent)] if urgent else [])
            + (["-a", ",".join(active)] if active else []),
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )

        proc.stdin.write(self._separator.join(formatted_notifications))
        proc.stdin.close()

        maybe_selection = (lambda x: int(x) if x else None)(
            proc.stdout.read().decode("utf-8")
        )
        operation = Operation(proc.wait())
        return maybe_selection, operation



#!/usr/bin/env python3
import logging

import i3ipc


def switch():
    def switch_to_urgent(i3, data):
        i3.main_quit()
        i3.command(f"workspace {data.current.name}")

    i3 = i3ipc.Connection(auto_reconnect=True)
    i3.on(i3ipc.Event.WORKSPACE_URGENT, switch_to_urgent)
    i3.main(3)


switch()
